import argparse
import sys
import pandas as pd
import numpy as np

def main():
	parser = argparse.ArgumentParser()
	parser.add_argument('-out', required=True)
	parser.add_argument('-input', required=False)
	parser.add_argument('-dep', required=False)
	parser.add_argument('-depcor', required=False)
	parser.add_argument('-onco_potential', required=False)
	parser.add_argument('-mut', required=False)
	parser.add_argument('-cnv', required=False)

	args = parser.parse_args()
	outfile = args.out

	# check parameters
	if not(args.dep or args.input):
		sys.exit( 'Need to input achilles dependency file or the compiled gene info file' )
#    if (args.mut and (not args.dep and not args.cnv)) or (args.cnv and (not args.dep and not args.mut):
#		sys.exit( 'mut, cnv, and dep files needs to be all present' )

	data = pd.DataFrame()

	# get the basis file	
	if args.input:
		infile = args.input
		data = loadFile(infile)
	elif args.dep:
		dep_file = args.dep
		data = loadDepScoreFile(dep_file)

	
	# start adding columns based on inputs
	if args.depcor:
		depCor_file = args.depcor
		data = data.join(minmaxDepCor(data.index, depCor_file), lsuffix='_1', rsuffix='_2')
	
	if args.onco_potential:
		oncopot_files = str(args.onco_potential).split(',')
		for i in range( len(oncopot_files) ):
			if '_og_' in oncopot_files[i]:
				data = data.join(oncoPotential(data.index, oncopot_files[i], 'OG'), lsuffix='_1', rsuffix='_2')
			elif '_tsg_' in oncopot_files[i]:
				data = data.join(oncoPotential(data.index, oncopot_files[i], 'TSG'), lsuffix='_1', rsuffix='_2')
	
	if args.mut and args.cnv:
		dep_file  = args.dep
		mut_files = args.mut
		cnv_file  = args.cnv
		data = data.join(varDS(data.index, dep_file, mut_files, cnv_file), lsuffix='_1', rsuffix='_2')
	
	data.index.name = 'gene'
	data.to_csv(outfile, sep='\t')

def varDS(genelist, dep_file, mut_files, cnv_file):
	print('loading var ds files...')
	dep = loadFile(dep_file)
	
	#load mutation
	wildtype = pd.DataFrame()
	damaging = pd.DataFrame()
	other = pd.DataFrame()
	
	files = mut_files.split(',')
	for i in range( len(files) ):
		if 'wildtype' in files[i]:
			wildtype = loadFile(files[i])
		if 'damaging' in files[i]:
			damaging = loadFile(files[i])
		if 'other' in files[i]:
			other = loadFile(files[i])
	
	print('reindexing mut file...')
	wildtype = wildtype.reindex(index=list(dep.index), columns=list(dep.columns) )
	damaging = damaging.reindex(index=list(dep.index), columns=list(dep.columns) )
	#other = other.reindex( columns=list(dep.columns) )


	#load cnv
	print('loading and reindexing cnv file...')
	cnv = loadFile(cnv_file)

	cnv = cnv.reindex( index=list(dep.index), columns=list(dep.columns) )


	print('condition selecting wt data...')
	dep_wt = dep[wildtype == 1]
	dep_wt = dep_wt[(cnv > 0.5) & (cnv < 2)]

	print('condition selecting mt data...')
	dep_mt = dep[damaging == 1]
	dep_mt_loss = dep[((cnv < 0.5))]
	dep_mt_loss.update(dep_mt)

	print('condition selecting mt data...')
	dep_mt = dep[damaging == 1]
	dep_mt_gain = dep[((cnv > 2))]
	dep_mt_gain.update(dep_mt)
	
	
	print(dep_mt_loss.shape)
	print(dep_mt_gain.shape)

	print('calculating mean and sd...')
	dep_var = pd.concat( [dep_wt.mean(axis=0), dep_wt.std(axis=0), dep_mt_loss.mean(axis=0), dep_mt_loss.std(axis=0), dep_mt_gain.mean(axis=0), dep_mt_gain.std(axis=0)], axis=1)
	dep_var.columns = ['wt_ds_mean', 'wt_ds_sd', 'mt_loss_ds_mean', 'mt_loss_ds_sd', 'mt_gain_ds_mean', 'mt_loss_ds_sd']
    
	return dep_var

def oncoPotential(genelist, f, filetype):
	oncopot = loadFile(f)

	oncopot = oncopot.reindex(index=list(genelist))

	df = pd.DataFrame(index=genelist)

	df[filetype+'_onco_potential'] = oncopot['TUSON_q_value_'+filetype]

	return df
	

def minmaxDepCor(genelist, f):
	depCor = loadFile(f)
	
	depCor = depCor.reindex(index=list(genelist))
		
	df = pd.DataFrame(index = genelist, columns=['max_ds_cor', 'max_ds_cor_gene'] )

	for gene in depCor.columns:
		df.at[gene, 'max_ds_cor'] = depCor[gene].nlargest(2)[1]
		df.at[gene, 'max_ds_cor_gene'] = depCor[gene].nlargest(2).index[1]

	df['min_ds_cor'] = depCor.min(axis=0)
	df['min_ds_cor_gene'] = depCor.idxmin(axis=0)

	return df

	
def loadDepScoreFile(f):
	dep = loadFile(f)

	data = pd.DataFrame(index=dep.columns)

	data['ds_mean'] = dep.mean(axis=0)
	data['ds_sd'] = dep.std(axis=0)

	return data

def loadFile(f):
	df = pd.DataFrame()
	if f.endswith('pkl'):
		df = pd.read_pickle(f)
	elif f.endswith('tsv') or f.endswith('txt'):
		df = pd.read_csv(f, header=0, index_col=0, sep='\t' )
	elif f.endswith('csv'):
		df = pd.read_csv(f, header=0, index_col=0)

	df.index.name = 'gene'

	return df

if __name__ == "__main__":
	main()
